#include "tsbrowse3.h"
#pragma hdrstop

#include "tsb3_f_resourcefile.h"
#include "tsb3_f_rentry.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TResourceFileForm *ResourceFileForm;
//---------------------------------------------------------------------------


// ADDICONRESOURCE and ADDBITMAPRESOURCE
// These will add resources to the project. You must save the project module yourself.
// You must Release() the TIResourceEntry that is return to you.
// AddIconResource returns the resource entry for the first icon in the group. If
// you gave it a non-null groupre, then that will return the TIResourceEntry for
// the icon group: you must Release() that as well. If you gave it a null groupre,
// then no need to release it.
TIResourceEntry* __fastcall AddIconResource(AnsiString fn, char *name,WORD flags,WORD langid,int DataVersion,int Version,int Characteristics,TIResourceEntry**groupre);
TIResourceEntry* __fastcall AddBitmapResource(AnsiString fn,char *name,WORD flags,WORD langid,int DataVersion,int Version,int Characteristics);

// The following code is from Lischner ch. 16. He says:
// Note that the data format in a file differs from the data
// format in the resource. The ResInfo unit declares the format
// for resource files. The declarations below pertain to
// external .ICO and .CUR files.
#pragma pack(push,1)
typedef struct tagFileGroupItem
{ BYTE bWidth;
  BYTE bHeight;
  BYTE bColorCount;
  BYTE bReserved;
  WORD wPlanes; WORD wBitCount;
  DWORD dwSize; // item size in bytes
  DWORD dwOffset; // start of item in the file
} TFileGroupItem;

typedef struct tagFileGroupHeader
{ WORD wReserved;
  WORD wType; // 1 for icons, 2 for cursors
  WORD wCount;
  TFileGroupItem Items[1];
} TFileGroupHeader;

typedef struct tagGroupItem
{ BYTE bWidth;
  BYTE bHeight;
  BYTE bColorCount;
  BYTE bReserved;
  WORD wPlanes;
  WORD wBitCount;
  DWORD dwBytesInRes;
  WORD wNameOrdinal;
} TGroupItem;

typedef struct tagGroupHeader
{ WORD wReserved;
  WORD wType; // 1 for icons, 2 for cursors
  WORD wCount;
  TGroupItem Items[1];
} TGroupHeader;

#pragma pack(pop)



TIResourceEntry* __fastcall AddIconResource(AnsiString fn, char *name,WORD flags,WORD langid,int DataVersion,int Version,int Characteristics,TIResourceEntry**groupre)
{ if (!FileExists(fn)) return NULL;
  TIModuleInterface *mi=ToolServices->GetModuleInterface(ToolServices->GetProjectName()); if (mi==NULL) return NULL;
  TIResourceFile *rf=mi->GetProjectResource(); if (rf==NULL) return NULL;
  // Lischner: Create an icon or cursor resource by separating the group header
  // and creating a separate resource for each item in the group. Then create a
  // group entry. Automatically assign a resource identifier for each individual
  // resource, ensuring that the identifiers do not conflict with any identifiers already in use.
  // First determine the highest resource ID currently assigned to a resource of this type
  int MaxId=0;
  for (int i=0; i<rf->GetEntryCount(); i++)
  { TIResourceEntry *te=rf->GetEntry(i);
    if (te->GetResourceType()==RT_ICON)
    { int resid=(int)te->GetResourceName();
      if (HIWORD(resid)==0 && resid>MaxId) MaxId=resid;
    }
    te->Release();
  }
  // Assign MaxId+1 as the ID of the first image resource.
  MaxId++;
  // Read the entire file into a memory stream.
  TMemoryStream *MemStream=new TMemoryStream();
  TFileStream *InStream=new TFileStream(fn,fmOpenRead|fmShareDenyWrite);
  MemStream->CopyFrom(InStream, 0);
  delete InStream;
  // Extract each icon/cursor image.
  TFileGroupHeader *FileHdr = (TFileGroupHeader*)MemStream->Memory;
  int ResHdrSize = 3*sizeof(WORD) + FileHdr->wCount*sizeof(TGroupItem);
  TGroupHeader *ResHdr=(TGroupHeader*)new char[ResHdrSize];
  ResHdr->wReserved=0;
  ResHdr->wType=FileHdr->wType;
  ResHdr->wCount=FileHdr->wCount;
  TIResourceEntry *openre=NULL;
  for (int i=0; i<FileHdr->wCount; i++)
  { CopyMemory(&(ResHdr->Items[i]),&(FileHdr->Items[i]),sizeof(TGroupItem));
    ResHdr->Items[i].wNameOrdinal=(WORD)MaxId;
    TIResourceEntry *re=rf->CreateEntry(RT_ICON,MAKEINTRESOURCE(MaxId),flags,langid,DataVersion,Version,Characteristics);
    re->SetDataSize(ResHdr->Items[i].dwBytesInRes);
    TResourceDataStream *OutStream=new TResourceDataStream(re);
    MemStream->Position = FileHdr->Items[i].dwOffset;
    OutStream->CopyFrom(MemStream, FileHdr->Items[i].dwSize);
    delete OutStream;
    if (openre==NULL) openre=re; else re->Release();
    MaxId++;
  }
  // Create the group resource entry.
  TIResourceEntry *re=rf->CreateEntry(MAKEINTRESOURCE(RT_GROUP_ICON),name,flags,langid,DataVersion,Version,Characteristics);
  re->SetDataSize(ResHdrSize);
  TResourceDataStream *OutStream=new TResourceDataStream(re);
  OutStream->Write(ResHdr,ResHdrSize);
  delete OutStream;
  if (groupre==NULL) re->Release(); else *groupre=re;
  delete[] ResHdr;
  delete MemStream;
  rf->Release();
  mi->Release();
  return openre;
}


TIResourceEntry* __fastcall AddBitmapResource(AnsiString fn,char *name,WORD flags,WORD langid,int DataVersion,int Version,int Characteristics)
{ if (!FileExists(fn)) return NULL;
  TIModuleInterface *mi=ToolServices->GetModuleInterface(ToolServices->GetProjectName()); if (mi==NULL) return NULL;
  TIResourceFile *rf=mi->GetProjectResource(); if (rf==NULL) return NULL;
  // Lischner: When storing a bitmap in a resource file we do not store the
  // bitmap file header that exists in the disk file.
  TIResourceEntry *re=rf->CreateEntry(RT_BITMAP,name,flags,langid,DataVersion,Version,Characteristics);
  TFileStream *fs=new TFileStream(fn,fmOpenRead | fmShareDenyWrite);
  re->SetDataSize(fs->Size-sizeof(TBitmapFileHeader));
  fs->Seek(sizeof(TBitmapFileHeader),soFromBeginning);
  TResourceDataStream *ds=new TResourceDataStream(re);
  ds->CopyFrom(fs,fs->Size-fs->Position);
  delete ds;
  delete fs;
  rf->Release();
  mi->Release();
  return re;
}









//---------------------------------------------------------------------------
//---------------------------------------------------------------------------


__fastcall TResourceFileForm::TResourceFileForm(TComponent* Owner)
    : TForm(Owner)
{ eCreateLang->Text="0x"+IntToHex((int)GetUserDefaultLangID(),8);
}
//---------------------------------------------------------------------------

void __fastcall TResourceFileForm::btnFileNameClick(TObject *Sender)
{ tFileName->Caption=rf->FileName();
}
//---------------------------------------------------------------------------

void __fastcall TResourceFileForm::btnGetEntryCountClick(TObject *Sender)
{ tGetEntryCount->Caption=AnsiString(rf->GetEntryCount());
}
//---------------------------------------------------------------------------


void __fastcall TResourceFileForm::FormCreate(TObject *Sender)
{ PageControl->ActivePage=tsGeneral;
}
//---------------------------------------------------------------------------

void __fastcall TResourceFileForm::btnGetEntryClick(TObject *Sender)
{ int i; try {i=eGetEntry->Text.ToInt();} catch (EConvertError &err) {MessageBeep(0);return;}
  TIResourceEntry *re=rf->GetEntry(i);
  if (re==NULL)
  { Application->MessageBox("Resource entry not found","Error",MB_OK);
  }
  else
  { TResourceEntryForm *f=new TResourceEntryForm(this);
    f->re=re;
    f->ShowModal();
    delete f;
    re->Release();
  }
}
//---------------------------------------------------------------------------

void __fastcall TResourceFileForm::btnGetEntryFromHandleClick(
      TObject *Sender)
{ int i; try {i=eGetEntryFromHandle->Text.ToInt();} catch (EConvertError &err) {MessageBeep(0);return;}
  TIResourceEntry *re=rf->GetEntryFromHandle((void*)i);
  if (re==NULL)
  { Application->MessageBox("Resource entry handle not found","Error",MB_OK);
  }
  else
  { TResourceEntryForm *f=new TResourceEntryForm(this);
    f->re=re;
    f->ShowModal();
    delete f;
    re->Release();
  }
}
//---------------------------------------------------------------------------

void __fastcall TResourceFileForm::btnDeleteHandleClick(TObject *Sender)
{ int i; try {i=eGetEntryFromHandle->Text.ToInt();} catch (EConvertError &err) {MessageBeep(0);return;}
  rf->DeleteEntry((void*)i);
}
//---------------------------------------------------------------------------

void __fastcall TResourceFileForm::btnFindEntryClick(TObject *Sender)
{ char *name=ResourceNameFromString(eFindEntry->Text);
  char *type=ResourceTypeFromString(cbFindEntry->Text);
  if (type==NULL || name==NULL) {MessageBeep(0);return;}
  TIResourceEntry *re=rf->FindEntry(type,name);
  if (re==NULL)
  { Application->MessageBox("Resource entry not found","Error",MB_OK);
  }
  else
  { TResourceEntryForm *f=new TResourceEntryForm(this);
    f->re=re;
    f->ShowModal();
    delete f;
    re->Release();
  }
}
//---------------------------------------------------------------------------
TIResourceEntry* __fastcall TResourceFileForm::CreateEntry()
{ char *type=ResourceTypeFromString(cbCreateType->Text);
  char *name=ResourceNameFromString(eCreateName->Text.c_str());
  if (type==NULL||name==NULL) return NULL;
  WORD flags,langid; int DataVersion,Version,Characteristics;
  try
  { flags=(WORD)eCreateFlags->Text.ToInt();
    langid=(WORD)eCreateLang->Text.ToInt();
    DataVersion=eCreateDataVersion->Text.ToInt();
    Version=eCreateVersion->Text.ToInt();
    Characteristics=eCreateCharacteristics->Text.ToInt();
  }
  catch (EConvertError &err) {return NULL;}
  TIResourceEntry *re=rf->CreateEntry(type,name,flags,langid,DataVersion,Version,Characteristics);
  return re;
}

void __fastcall TResourceFileForm::btnCreateClick(TObject *Sender)
{ TIResourceEntry *re=CreateEntry();
  if (re==NULL)
  { Application->MessageBox("Failed to create resource entry","Error",MB_OK);
  }
  else
  { TResourceEntryForm *f=new TResourceEntryForm(this);
    f->re=re;
    f->ShowModal();
    delete f;
    re->Release();
  }
}
//---------------------------------------------------------------------------

void __fastcall TResourceFileForm::btnCreateBitmapClick(TObject *Sender)
{ cbCreateType->Text="RT_BITMAP";
  char *type=ResourceTypeFromString(cbCreateType->Text);
  char *name=ResourceNameFromString(eCreateName->Text.c_str());
  if (type==NULL||name==NULL) return;
  WORD flags,langid; int DataVersion,Version,Characteristics;
  try
  { flags=(WORD)eCreateFlags->Text.ToInt();
    langid=(WORD)eCreateLang->Text.ToInt();
    DataVersion=eCreateDataVersion->Text.ToInt();
    Version=eCreateVersion->Text.ToInt();
    Characteristics=eCreateCharacteristics->Text.ToInt();
  }
  catch (EConvertError &err) {return;}
  if (ExtractFileExt(CreateDialog1->FileName)!=".ico") CreateDialog1->FileName="";
  CreateDialog1->DefaultExt="bmp";
  CreateDialog1->Filter="Bitmaps (*.bmp)|*.bmp";
  bool res=CreateDialog1->Execute();
  if (!res) return;
  TIResourceEntry *re=AddBitmapResource(CreateDialog1->FileName,name,flags,langid,DataVersion,Version,Characteristics);
  if (re==NULL) {Application->MessageBox("Failed to create bitmap resource","Error",MB_OK);return;}
  TResourceEntryForm *f=new TResourceEntryForm(this);
  f->re=re;
  f->ShowModal();
  delete f;
  re->Release();
}
//---------------------------------------------------------------------------



void __fastcall TResourceFileForm::btnCreateIconClick(TObject *Sender)
{ cbCreateType->Text="RT_ICON";
  char *type=ResourceTypeFromString(cbCreateType->Text);
  char *name=ResourceNameFromString(eCreateName->Text.c_str());
  if (type==NULL||name==NULL) return;
  WORD flags,langid; int DataVersion,Version,Characteristics;
  try
  { flags=(WORD)eCreateFlags->Text.ToInt();
    langid=(WORD)eCreateLang->Text.ToInt();
    DataVersion=eCreateDataVersion->Text.ToInt();
    Version=eCreateVersion->Text.ToInt();
    Characteristics=eCreateCharacteristics->Text.ToInt();
  }
  catch (EConvertError &err) {return;}
  if (ExtractFileExt(CreateDialog1->FileName)!=".ico") CreateDialog1->FileName="";
  CreateDialog1->DefaultExt="ico";
  CreateDialog1->Filter="Icons (*.ico)|*.ico";
  bool res=CreateDialog1->Execute();
  if (!res) return;
  TIResourceEntry *re=AddIconResource(CreateDialog1->FileName,name,flags,langid,DataVersion,Version,Characteristics,NULL);
  if (re==NULL) {Application->MessageBox("Failed to create icon resource","Error",MB_OK);return;}
  TResourceEntryForm *f=new TResourceEntryForm(this);
  f->re=re;
  f->ShowModal();
  delete f;
  re->Release();
}
//---------------------------------------------------------------------------




